#!/usr/bin/env python

from practicum import findDevices
from peri import PeriBoard
from time import sleep
from math import *
from numpy import interp
from usb.core import USBError

import random, os.path
from random import randint

#import basic pygame modules
import pygame
from pygame.locals import *

devs = findDevices()
if len(devs) == 0:
    print '*** No MCU board found.'
    b = None
else:
    b = PeriBoard(devs[0])
    print '*** MCU board found'
    print '*** Device manufacturer: %s' % b.getVendorName()
    print '*** Device name: %s' % b.getDeviceName()
    print 'Calibrating board.'
    raw_input('Face the board down then press ENTER...')
    minLight = b.getLight()
    raw_input('Face the board up then press ENTER...')
    maxLight = b.getLight()
    b.lightRange = (minLight,maxLight)
    print 'Calibration done.  Light range is (%d,%d)' % b.lightRange



#see if we can load more than standard BMP
if not pygame.image.get_extended():
    raise SystemExit("Sorry, extended image module required")


#game constants
MAX_SHOTS      = 10      
SCREENRECT     = Rect(0, 0, 1200, 600)
SCORE          = 0
SMOOTH_FACTOR  = 0.2
R_FROM_PLAYER = 100


main_dir = os.path.split(os.path.abspath(__file__))[0]

def load_image(file):
    "loads an image, prepares it for play"
    file = os.path.join(main_dir, 'data', file)
    try:
        surface = pygame.image.load(file)
    except pygame.error:
        raise SystemExit('Could not load image "%s" %s'%(file, pygame.get_error()))
    return surface.convert()

def load_images(*files):
    imgs = []
    for file in files:
        imgs.append(load_image(file))
    return imgs


class dummysound:
    def play(self): pass

def load_sound(file):
    if not pygame.mixer: return dummysound()
    file = os.path.join(main_dir, 'data', file)
    try:
        sound = pygame.mixer.Sound(file)
        return sound
    except pygame.error:
        print ('Warning, unable to load, %s' % file)
    return dummysound()


class Player(pygame.sprite.Sprite):
    speed = 10
    bounce = 24
    gun_offset = -11
    FL = 160 #FirstLocationY
    SL = 400 #SecondLocationY
    images = []
    def __init__(self):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.rect = self.image.get_rect(midbottom=SCREENRECT.midbottom)
        #self.rect = self.image.get_rect(10,self.FL)
        self.rect.top = self.FL - (self.rect.height / 2)
        self.rect.left = 100
        self.reloading = 0
        #self.origtop = self.rect.top
        self.facing = -1

    def move(self, direction):
        A = 2

    #def gunpos(self):
        #pos = self.facing*self.gun_offset + self.rect.centerx
        #return pos, self.rect.top
        #return self.rect.center

class AngleIndicator(pygame.sprite.Sprite):
    images = []
    oAngle = radians(-45)
    def __init__(self,controller,playerRef):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.rect = self.image.get_rect()
        self.controller = controller
        self.ref = playerRef

    def update(self):
        if self.controller != None:
            try:
                light = self.controller.getLight()
                angle = self.oAngle + interp(light,self.controller.lightRange,[0,radians(90)])
                self.rotate(angle)
            except USBError:
                print "USB error detected"

    def rotate(self,angle):
        self.dirX = cos(angle)
        self.dirY = sin(angle)
        self.rect.centerx = SMOOTH_FACTOR*(self.ref.rect.centerx + R_FROM_PLAYER*self.dirX) + (1-SMOOTH_FACTOR)*self.rect.centerx
        self.rect.centery = SMOOTH_FACTOR*(self.ref.rect.centery + R_FROM_PLAYER*self.dirY) + (1-SMOOTH_FACTOR)*self.rect.centery


class PussyCatExploder(pygame.sprite.Sprite):
    speed = 8
    animcycle = 12
    images = []
    def __init__(self):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.frame = 0
        self.rect = self.image.get_rect()
        self.rect.topleft = (SCREENRECT.width, random.random() * (SCREENRECT.height-self.image.get_height()))

    def update(self):
        self.rect.move_ip(-self.speed, 0)
        if self.rect.top <= -100:
            self.kill()
        if self.rect.bottom >= 1000: ##
            self.kill()
        if self.rect.left <= 255:
            self.kill()
        if self.rect.right >= 1500: ##
            self.kill()
        self.frame = self.frame + 1

class HealthFlyer(pygame.sprite.Sprite):
    speed = 8
    animcycle = 12
    images = []
    def __init__(self):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.frame = 0
        self.rect = self.image.get_rect()
        self.rect.topleft = (SCREENRECT.width, random.random() * (SCREENRECT.height-self.image.get_height()))

    def update(self):
        self.rect.move_ip(-self.speed, 0)
        if self.rect.top <= -100:
            self.kill()
        if self.rect.bottom >= 1000: ##
            self.kill()
        if self.rect.left <= 255:
            self.kill()
        if self.rect.right >= 1500: ##
            self.kill()
        self.frame = self.frame + 1

class Rival(pygame.sprite.Sprite):
    speed = 0
    yspeed = 0
    animcycle = 12
    hit = 0
    images = []
    def __init__(self):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        #self.facing = random.choice((-1,1)) * Alien.speed
        self.frame = 0
        #if self.facing < 0:
            #self.rect.top = SCREENRECT.top

class Grammy(Rival):
    def __init__(self,posx = SCREENRECT.width,posy=random.random() * (SCREENRECT.height-64)):
        super(Grammy,self).__init__()
        self.rect = self.image.get_rect()
        self.rect.topleft = (posx,posy)
        self.speed += 1
        self.yspeed = self.yspeed + randint(1,4)

    def update(self):
        self.rect.move_ip(-self.speed, self.yspeed)
        if self.rect.top <= 0:
            self.yspeed = abs(self.yspeed)
            self.rect.move_ip(-self.speed, self.yspeed)
        if self.rect.bottom >= 600: ##
            self.yspeed = -1 * abs(self.yspeed)
            self.rect.move_ip(-self.speed, self.yspeed)
        if self.rect.left <= -100:
            self.kill()
        if self.rect.right >= 1500: ##
            self.kill()
        self.frame = self.frame + 1

class Taylor(Rival):
    def __init__(self):
        super(Taylor,self).__init__()
        self.rect = self.image.get_rect()
        self.rect.topleft = (SCREENRECT.width, random.random() * (SCREENRECT.height-self.image.get_height()))
        self.speed += 3

    def update(self):
        self.rect.move_ip(-self.speed, 0)
        self.frame = self.frame + 1

class Adele(Rival):
    def __init__(self):
        super(Adele,self).__init__()
        self.image = pygame.transform.scale(self.image,(2*self.image.get_width(),2*self.image.get_height()))
        self.rect = self.image.get_rect(bottomleft=SCREENRECT.bottomright)
        #self.rect.left = 1100
        #self.rect.top = randint(0,400)
        self.speed = 1

    def update(self):
        self.rect.move_ip(-self.speed, 0)
        self.frame = self.frame + 1


class Explosion(pygame.sprite.Sprite):
    defaultlife = 12
    animcycle = 3
    images = []
    def __init__(self, actor):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.rect = self.image.get_rect(center=actor.rect.center)
        self.life = self.defaultlife

    def update(self):
        self.life = self.life - 1
        self.image = self.images[self.life//self.animcycle%2]
        if self.life <= 0: self.kill()


class Shot(pygame.sprite.Sprite):
    speed = 12
    xval = 0
    yval = 0
    images = []
    def __init__(self,centerx,centery, xval, yval):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.rect = self.image.get_rect()
        self.rect.centerx = centerx
        self.rect.centery = centery
        self.xval = xval
        self.yval = yval
        #self.rect = self.image.get_rect(midbottom=pos)

    def update(self):
        self.rect.move_ip(self.xval,self.yval)
        if self.rect.top <= 0:
            self.kill()
        if self.rect.bottom >= 600: ##
            self.kill()
        if self.rect.left <= 0:
            self.kill()
        if self.rect.right >= 1200: ##
            self.kill()


class Bomb(pygame.sprite.Sprite):
    speed = 9
    images = []
    def __init__(self, alien):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.rect = self.image.get_rect(midbottom=
                    alien.rect.move(0,5).midbottom)

    def update(self):
        self.rect.move_ip(-self.speed, 0)
        if self.rect.left <= 0:
            Explosion(self)
            self.kill()


class Score(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.font = pygame.font.Font(None, 40)
        self.font.set_bold(1)
        self.color = Color('black')
        self.lastscore = -1
        self.update()
        self.rect = self.image.get_rect().move(10, SCREENRECT.height - 30)

    def update(self):
        if SCORE != self.lastscore:
            self.lastscore = SCORE
            msg = "Score: %d" % SCORE
            self.image = self.font.render(msg, 0, self.color)


def main(winstyle = 0):
    # Initialize pygame
    pygame.mixer.pre_init(44100, -16, 2, 2048)
    pygame.init()
    if pygame.mixer and not pygame.mixer.get_init():
        print ('Warning, no sound')
        pygame.mixer = None

    # Set the display mode
    winstyle = 0  # |FULLSCREEN
    bestdepth = pygame.display.mode_ok(SCREENRECT.size, winstyle, 32)
    screen = pygame.display.set_mode(SCREENRECT.size, winstyle, bestdepth)

    #Load images, assign to sprite classes
    #(do this before the classes are used, after screen setup)
    img = load_image('Player_Katy.gif')
    Player.images = [img, pygame.transform.flip(img, 1, 0)]
    img = load_image('explosion1.gif')
    Explosion.images = [img, pygame.transform.flip(img, 1, 1)]
    Grammy.images = [load_image('Mon_Grammy.gif')]
    Taylor.images = [load_image('Mon_Taylor.gif')]
    Adele.images = [load_image('Mon_Adele.gif')]
    Shot.images = [load_image('pussycat.gif')]
    AngleIndicator.images = [load_image('pussycatmini.gif')]
    PussyCatExploder.images = [load_image('PussyCatExplode.gif')]
    HealthFlyer.images = [load_image('Heart.gif')]

    #decorate the game window
    icon = pygame.transform.scale(Grammy.images[0], (32, 32))
    pygame.display.set_icon(icon)
    pygame.display.set_caption('Katy The No-Grammy')
    pygame.mouse.set_visible(0)

    #create the background, tile the bgd image
    bgdtile = load_image('bg.gif')
    background = pygame.Surface(SCREENRECT.size)
    for x in range(0, SCREENRECT.width, bgdtile.get_width()):
        background.blit(bgdtile, (x, 0))
    screen.blit(background, (0,0))
    pygame.display.flip()

    #load the sound effects
    boom_sound = load_sound('boom.wav')
    #adele_sound = load_sound('Adele_Hello.wav') Disable due to license issue
    #taylor_sound = load_sound('Bad_Blood.wav') Disable due to license issue
    cat_shot_sound = load_sound('Cat2.wav')
    cat_hit_sound = load_sound('Cat3.wav')
    if pygame.mixer:
        music = os.path.join(main_dir, 'data', 'gamemusic.wav')
        pygame.mixer.music.set_volume(1.0)
        pygame.mixer.music.load(music)
        pygame.mixer.music.play(-1)

    # Initialize Game Groups
    rivals = pygame.sprite.Group()
    shots = pygame.sprite.Group()
    bombs = pygame.sprite.Group()
    pussycatexploders = pygame.sprite.Group()
    healthflyers = pygame.sprite.Group()
    all = pygame.sprite.RenderUpdates()

    lastrival = pygame.sprite.GroupSingle()

    #assign default groups to each sprite class
    Player.containers = all
    Grammy.containers =  rivals, all, lastrival
    Taylor.containers = rivals, all, lastrival
    Adele.containers = rivals, all, lastrival
    Shot.containers = shots, all
    PussyCatExploder.containers = pussycatexploders, all
    Bomb.containers = bombs, all
    Explosion.containers = all
    Score.containers = all
    AngleIndicator.containers = all
    HealthFlyer.containers = healthflyers, all

    #Create Some Starting Values
    global score
    kills = 0
    clock = pygame.time.Clock()

    #initialize our starting sprites
    global SCORE
    player = Player()
    ang = AngleIndicator(b,player)
    s = Score()
    if pygame.font:
        all.add(s)

    framepasscount = 1
    nextstateframe = framepasscount - 1 #ForSwitching
    nextstateframe2 = framepasscount - 1 #ForAngelSenting
    nextstateframe3 = framepasscount - 1 #ForShooting
    nextstateframe4 = framepasscount - 1 #ForSpawningMonster
    HardRate = 1
    isHarder = False

    # PreProbability Configuration !! Must Change In The MonSpawner Section To the same value as this
    GrammyChance = 90 ## Percent (ecc. 70/100)
    TaylorChance = 50
    AdeleChance = 10
    PussyCatExploderChance = 20
    HealthFlyerChance = 40

    # HealthConfiguration
    PlayerHealth = 8 #Maximum 8
    HealthUpper = 1

    if PlayerHealth <= 3:
        b.setLed(0,1)
        b.setLed(1,0)
        b.setLed(2,0)
    elif PlayerHealth <= 6:
        b.setLed(0,0)
        b.setLed(1,1)
        b.setLed(2,0)
    else:
        b.setLed(0,0)
        b.setLed(1,0)
        b.setLed(2,1)


    while player.alive():
        framepasscount = framepasscount + 1
        #get input
        for event in pygame.event.get():
            if event.type == QUIT or \
                (event.type == KEYDOWN and event.key == K_ESCAPE):
                    b.setLed(0,0)
                    b.setLed(1,0)
                    b.setLed(2,0)
                    return
        keystate = pygame.key.get_pressed()

        # clear/erase the last drawn sprites
        all.clear(screen, background)

        #update all the sprites
        all.update()

        #handle player input
        if framepasscount > nextstateframe:
            try:
                sw = b.getSwitch()
                if sw is True:
                    nextstateframe = framepasscount + 10
                    if player.rect.top < 200:
                        player.rect.top = player.SL - (player.rect.height / 2)
                    else:
                        player.rect.top = player.FL - (player.rect.height / 2)
            except USBError:
                print "USB Error Detected"

        firing = 0
        if framepasscount > nextstateframe3:
            firing = 1
            reducefiringTime = HardRate * 10
            if reducefiringTime > 80:
                reducefiringTime = 80
            nextstateframe3 = nextstateframe3 + 100 - reducefiringTime

        if framepasscount % 1000 == 0 :
            print "The Game Get Harder Than Before"
            HardRate += 1
            Rival.speed += 1
            isHarder = True


        if not player.reloading and firing and len(shots) < MAX_SHOTS:
            Shot( player.rect.centerx, player.rect.centery, Shot.speed*ang.dirX, Shot.speed*ang.dirY)
            cat_shot_sound.play()
        player.reloading = firing

        # Create new alien



        if framepasscount > nextstateframe4: #SpawningMonsterAndHelper

            #ChanceHardRateMultiply = HardRate / 10.0
            Grammy_random = (randint(1,100) <= GrammyChance)
            Taylor_random = (randint(1,100) <= TaylorChance)
            Adele_random = (randint(1,100) <= AdeleChance)
            PussyCatExploder_random = (randint(1,100) <= PussyCatExploderChance)
            HealthFlyer_random = (randint(1,100) <= HealthFlyerChance)

            if Grammy_random:
                Grammy()
            if Taylor_random:
                Taylor()
            if Adele_random:
                Adele()
                #adele_sound.play() Disable due to license issue
            if PussyCatExploder_random:
                PussyCatExploder()
            if HealthFlyer_random:
                HealthFlyer()

            if reducefiringTime < 250:
                reducefiringTime = HardRate * 10
            if isHarder :
                if GrammyChance < 90:
                    GrammyChance += 10
                if TaylorChance < 70:    #  Percent (ecc. 70/100)
                    TaylorChance += 10
                if AdeleChance < 50:
                    AdeleChance += 10
                if PussyCatExploderChance < 60:
                    PussyCatExploderChance += 10
                if HealthFlyerChance < 60:
                    HealthFlyerChance += 10
                isHarder = False
            nextstateframe4 = nextstateframe4 + 300 - reducefiringTime


        if framepasscount > 1000:
            nextstateframe = 1 + (nextstateframe - framepasscount)
            nextstateframe2 = 1 + (nextstateframe2 - framepasscount)
            nextstateframe3 = 1 + (nextstateframe3 - framepasscount)
            nextstateframe4 = 1 + (nextstateframe4 - framepasscount)
            framepasscount = 1

        # Detect collisions

        for rival in pygame.sprite.groupcollide(rivals, shots, 0, 1).keys():
            Explosion(rival)
            rival.hit += 1
            if isinstance(rival,Grammy):
                cat_hit_sound.play()
                rival.kill()
                SCORE = SCORE + 1
            elif isinstance(rival,Taylor):
                #taylor_sound.play() Disable due to license issue
                rival.kill()
                SCORE = SCORE + 2
            elif isinstance(rival,Adele):
                cat_hit_sound.play()
                if(rival.hit >= 3):
                    Grammy(rival.rect.centerx,rival.rect.centery)
                    Grammy(rival.rect.centerx,rival.rect.centery-50)
                    Grammy(rival.rect.centerx,rival.rect.centery+50)
                    rival.kill()
                    SCORE = SCORE + 5

        for pussy in pygame.sprite.groupcollide(pussycatexploders, shots, 1, 1).keys():
            cat_hit_sound.play()
            Shot( pussy.rect.centerx, pussy.rect.centery, 0, 5)
            Shot( pussy.rect.centerx, pussy.rect.centery, 5, 5)
            Shot( pussy.rect.centerx, pussy.rect.centery, 5, 0)
            Shot( pussy.rect.centerx, pussy.rect.centery, -5, 0)
            Shot( pussy.rect.centerx, pussy.rect.centery, -5, -5)
            Shot( pussy.rect.centerx, pussy.rect.centery, 0, -5)
            Shot( pussy.rect.centerx, pussy.rect.centery, -5, 5)
            Explosion(pussy)

        for healthflyer in pygame.sprite.groupcollide(shots, healthflyers, 1, 1).keys():
            cat_hit_sound.play()
            PlayerHealth = PlayerHealth + HealthUpper
            if PlayerHealth <= 3:
                b.setLed(0,1)
                b.setLed(1,0)
                b.setLed(2,0)
            elif PlayerHealth <= 6:
                b.setLed(0,0)
                b.setLed(1,1)
                b.setLed(2,0)
            else:
                b.setLed(0,0)
                b.setLed(1,0)
                b.setLed(2,1)

        for rival in rivals:
            if(rival.rect.left<=255):
                boom_sound.play()
                Explosion(rival)
                rival.kill()
                if isinstance(rival,Grammy):
                    PlayerHealth = PlayerHealth - 1
                elif isinstance(rival,Taylor):
                    PlayerHealth = PlayerHealth - 2
                elif isinstance(rival,Adele):
                    PlayerHealth = 0
                if PlayerHealth <= 3:
                    b.setLed(0,1)
                    b.setLed(1,0)
                    b.setLed(2,0)
                elif PlayerHealth <= 6:
                    b.setLed(0,0)
                    b.setLed(1,1)
                    b.setLed(2,0)
                else:
                    b.setLed(0,0)
                    b.setLed(1,0)
                    b.setLed(2,1)
            if(PlayerHealth <= 0):
                Explosion(player)
                player.kill()

        # Prevent the health not to go beyond MAX
        if PlayerHealth > 2:
            PlayerHealth = 2

        #draw the scene
        dirty = all.draw(screen)
        pygame.display.update(dirty)

        #cap the framerate
        clock.tick(40)

    if pygame.mixer:
        pygame.mixer.music.fadeout(1000)
    pygame.time.wait(3000)
    b.setLed(0,0)
    b.setLed(1,0)
    b.setLed(2,0)
    pygame.quit()



#call the "main" function if running this script
if __name__ == '__main__': main()
