from practicum import McuBoard

RQ_SET_LED    = 0
RQ_GET_SWITCH = 1
RQ_GET_LIGHT  = 2

####################################
class PeriBoard(McuBoard):

    ################################
    def setLed(self, led_no, led_val):
        '''
        Set status of LED led_no on peripheral board to led_val
        '''
        self.usbWrite(request=RQ_SET_LED,index=led_no,value=led_val)

    ################################
    def setLedValue(self, value):
        '''
        Display value's 3 LSBs on peripheral board's LEDs
        '''
        if(value&1>0):
            self.setLed(0,1)
        else:
            self.setLed(0,0)
        
        if(value&2>0):
            self.setLed(1,1)
        else:
            self.setLed(1,0)
        
        if(value&4>0):
            self.setLed(2,1)
        else:
            self.setLed(2,0)

    ################################
    def getSwitch(self):
        '''
        Return a boolean value indicating whether the switch on the peripheral
        board is currently pressed
        '''
        a = self.usbRead(request=RQ_GET_SWITCH)
        return bool(a[0])

    ################################
    def getLight(self):
        '''
        Return the current reading of light sensor on peripheral board
        '''
        a = self.usbRead(RQ_GET_LIGHT,length=2)
        return a[0]+a[1]*256
